use rapier2d::{dynamics::RigidBodySet, geometry::ColliderHandle};
use std::collections::HashMap;

use super::{
	super::{
		objects::{PhysSimShip, ShipControls},
		PhysStepResources,
	},
	ShipControllerStruct,
};

/// The Null controller is assigned to objects that are static or not controlled by the computer.
/// Most notably, the player's ship has a Null controller.
#[derive(Debug, Clone)]
pub struct NullShipController {}

impl NullShipController {
	/// Create a new ship controller
	pub fn new() -> Self {
		Self {}
	}
}

impl ShipControllerStruct for NullShipController {
	fn update_controls(
		&mut self,
		_res: &PhysStepResources,
		_rigid_bodies: &RigidBodySet,
		_ships: &HashMap<ColliderHandle, PhysSimShip>,
		_this_ship: ColliderHandle,
	) -> Option<ShipControls> {
		None
	}
}
